#Copyright (C) 2020 Samira Maghool and Marco Cremonini
#
#This file is part of the multiagent epidemic simulator developed by the research project described in the paper titled "A multicomponent model of awareness for different categories of network epidemics", by Samira Maghool, Nahid Maleki-Jirsaraei, and Marco Cremonini.
#
#    The multiagent epidemic simulator is free software: you can  
#    redistribute it and/or modify it under the terms of the GNU General
#    Public License as published by the Free Software Foundation, either
#    version 3 of the License, or (at your option) any later version.
#
#    The multiagent epidemic simulator is distributed in the hope that it 
#    will be useful, but WITHOUT ANY WARRANTY; without even the implied 
#    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#    See the GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with the multiagent epidemic simulator.  
#    If not, see <https://www.gnu.org/licenses/>.
        
''' This module is for running the agent-based simulation'''
''' @author: samiramaghool@gmail.com>'''

import os
import networkx as nx
from SimPy import Simulation as Sim
import NetworkLogger

#Constants
SUSCEPTIBLE = 0
INCUBATING_INFECTED = 1
MILD_INFECTED = 2
ACUTE_INFECTED = 3
CONTAINED = 4
RECOVERED_MI = 5
RECOVERED_AI = 6


        
class NetworkSimulation(Sim.Simulation):
    """Simulation support for agents in a complex network.
    
    Can run multiple fresh trials with the same input parameters. Writes system
    state evolution to file (states & network topologies)
    
    
    Optional parameters
    -------------------
    
    no_trials: the number of individual simulation trials 
    
    """

    def __init__(self, SIRSNet, SIRSAgent, SIRSStates, directory_name, NET_IMAGES_DIR, maxTime,
                 printGIF=False, no_trials= 200):

        
        Sim.Simulation.__init__(self)
        self.SIRSNet = SIRSNet
        self.SIRSAgent = SIRSAgent
        self.SIRSStates = SIRSStates
        self.directory_results = os.path.abspath(directory_name)
        self.directory_netDraw = os.path.abspath(NET_IMAGES_DIR)
        self.until = maxTime
        self.no_trials = no_trials
        self.printGIF = printGIF
        
        
    def runSimulation(self):
        print ("Starting simulation...")
      
        
        for i in range(self.no_trials):
            print "---Trial " + str(i) + " ---"
            self.runTrial(i)

        print ("Simulation completed. ")
        
    def runTrial(self, id):   
        self.initialize() 
               
        #set up agents'''
        print ("set up SIRS agents...")              
       
        for i in self.SIRSNet.nodes():
            agent = self.SIRSAgent.SIRSAgent(self.SIRSStates[i], (i, self, self.SIRSNet))
            self.SIRSNet.node[i]['agent'] = agent
            self.activate(agent, agent.Run())
            
        print ("set up logging...")
        #set up logging
        logging_interval = 1
        logger = NetworkLogger.NetworkLogger(self, self.directory_results, self.directory_netDraw, logging_interval, self.printGIF)
        self.activate(logger, logger.Run(), prior=True)
        
        #run simulation trial
        self.simulate(self.until)
        
        ''''write Files'''
        logger.logTrialToFiles(id)
        
       
        
